/*
	save configuration to file
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h> /* for strncpy */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>

#include "include/DefInputS.h"
#include "include/DefInputA.h"
#include "include/DefInputD.h"
#include "include/DefOutput.h"
#include "include/DefSpecial.h"
#include "include/DefTimer.h"
#include "include/DefCommon.h"
#include "include/DefConfig.h"
#include "include/DefIo.h"
#include "include/DefCommFunc.h"
#include "include/DefSwType.h"
#include "include/DefRadio.h"
#include "include/DefAudio.h"
#include "include/DefLightout.h"

#include "include/DefCommon.h"

//common
char *data;			//data from web page
char *date;			//data from web page
char dataTemp[100];
int16_t intTemp;
char fileName[100];
FILE *fp;
FILE *fp1;
char tempStr[20];
int len;

//record
CONFIG_WEB_DATA cwData;		//web config data


INPUT_STANDARD_DATA 	isData;			//standard input data
INPUT_AUXILIARY_DATA 	iaData;			//auxiliary input data
INPUT_DTMF_DATA 	idData;			//DTMF input data
OUTPUT_DATA 		opData;			//output data
TIMER_DATA 		trData;			//timer data
SPECT_FUNCTION_DATA 	sfData;			//special function data
IO_DEF_DATA             ioData[MAX_NUM_IO];     //io pins data
COMM_FUNC_DATA		commData;		//communication function data
SWITCH_TYPE_FUNC_DATA	swtypeData;		//switch type function data
RADIO_FUNC_DATA		radioData;		//radio function data
AUDIO_DATA		aoData;			//audio output data
SPECT_LIGHTOUT_DATA	lightoutData;		//lightout function data

void getAllConfig();		//get all configuration

short saveStandardInput();	//save standard input configuration
short saveAuxiliaryInput();	//save auxiliary input configuration
short saveOutput();		//save output configuration
short saveTimer();		//save timer configuration
short saveSpecial();		//save special function configuration
short saveIO();			//save IO pins configuration
short saveCommFunc();		//save communication function configuration
short saveSwtypeFunc();		//save switch type function configuration
short saveRadioFunc();		//save radio function configuration
short saveDTMFInput();		//save DTMF function configuration
short saveAudio();		//save audio configuration
short saveLightout();		//save lightout configuration

int main()
{
	memset(dataTemp, 0x0, sizeof(dataTemp));
	memset(&cwData, 0x0, sizeof(CONFIG_WEB_DATA));
	memset(&isData, 0x0, sizeof(INPUT_STANDARD_DATA));
	memset(&iaData, 0x0, sizeof(INPUT_AUXILIARY_DATA));
	memset(&idData, 0x0, sizeof(INPUT_DTMF_DATA));
	memset(&opData, 0x0, sizeof(OUTPUT_DATA));
	memset(&trData, 0x0, sizeof(TIMER_DATA));
	memset(&sfData, 0x0, sizeof(SPECT_FUNCTION_DATA));
	memset(&ioData, 0x0, sizeof(IO_DEF_DATA) * MAX_NUM_IO);
	memset(&commData, 0x0, sizeof(COMM_FUNC_DATA));
	memset(&swtypeData, 0x0, sizeof(SWITCH_TYPE_FUNC_DATA));
	memset(&radioData, 0x0, sizeof(RADIO_FUNC_DATA));
	memset(&aoData, 0x0, sizeof(AUDIO_DATA));
	memset(&lightoutData, 0x0, sizeof(SPECT_LIGHTOUT_DATA));

	printf("Content-Type: text/html\n\n");

	date = (char*)getenv("CONTENT_LENGTH");
	//len = atoi(getenv("CONTENT_LENGTH"));

	//read all configuration first
	getAllConfig();

	short result = 1;

	data = (char *)getenv("QUERY_STRING");

	if(data == NULL)
	{
		printf("fail");
	}
	else
	{
		sscanf(data, "type=%1s", dataTemp);
		if(dataTemp[0] == 'S')
		{
			//standard inputs
			result = saveStandardInput();
			if(result != 1)
			{
				printf("%d", result);
				return -1;
			}
		}
		else if(dataTemp[0] == 'A')
		{
			//auxiliary inputs
			result = saveAuxiliaryInput();
			if(result != 1)
			{
				printf("%d", result);
				return -1;
			}
		}
		else if(dataTemp[0] == 'D')
		{
			//DTMF inputs
			result = saveDTMFInput();
			if(result != 1)
			{
				printf("%d", result);
				return -1;
			}
		}
		else if(dataTemp[0] == 'O')
		{
			//outputs
			result = saveOutput();
			if(result != 1)
			{
				printf("%d", result);
				return -1;
			}
		}
		else if(dataTemp[0] == 'T')
		{
			//timer
			result = saveTimer();
			if(result != 1)
			{
				printf("%d", result);
				return -1;
			}
		}
		else if(dataTemp[0] == 'L')
		{
			//special
			result = saveSpecial();
			if(result != 1)
			{
				printf("%d", result);
				return -1;
			}
		}
		else if(dataTemp[0] == 'P')
		{
			//io pins
			result = saveIO();
			if(result != 1)
			{
				printf("%d", result);
				return -1;
			}
			/*printf("Special: p1=%c p2=%c p3=%c p4=%c p5=%lf p6=%lf\n",
				cwData.sfData.salone, cwData.sfData.swstatus, cwData.sfData.cpd, cwData.sfData.aureturn,
				cwData.sfData.bcaution, cwData.sfData.bdisable
			);*/
		}
		else if(dataTemp[0] == 'C')
		{
			//communication function
			result = saveCommFunc();
			if(result != 1)
			{
				printf("-5");
				return -1;
			}
		}
		else if(dataTemp[0] == 'B')
		{
			//switch type function
			result = saveSwtypeFunc();
			if(result != 1)
			{
				printf("-5");
				return -1;
			}
		}
		else if(dataTemp[0] == 'R')
		{
			//radio function
			result = saveRadioFunc();
			if(result != 1)
			{
				printf("-5");
				return -1;
			}
		}
		else if(dataTemp[0] == 'U')
		{
			result = saveAudio();
			if(result != 1)
			{
				printf("-5");
				return -1;
			}
		}
		else if(dataTemp[0] == 'V')
		{
			result = saveLightout();
			if(result != 1)
			{
				printf("-5");
				return -1;
			}
			else
			{
				printf("1");
				return -1;
			}
		}
		
		if(dataTemp[0] != 'V')
		{
			FILE *fp;
			short dataLen = sizeof(CONFIG_WEB_DATA);
			cwData.dataLen = dataLen;
			if((fp = fopen("/home/flexyard/flexyard/sysconfig/WebConfig.dat", "w")) == NULL)
			{
				printf("-1");//open fail
				return -1;
			}
			else
			{
				if(fwrite((char *)&cwData, sizeof(cwData), 1, fp) != 1)
				{
					printf("-2");//write fail
				}
				else
					printf("1");//success
			}
			fclose(fp);
		}
	}

	//printf("%s\n", "test123");

	return 0;
}

/*******************************************************************************
  Class :
  Function Name : saveStandardInput
  Input         :
  Output        :
  purpose       : save standard input config
  Note          :
*******************************************************************************/
short saveStandardInput()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p1"))
		{
			p = strtok(NULL, "&=");
			isData.sps = p[0];
			if(isData.sps != 'N' && isData.sps != 'S' && isData.sps != 'D')
			{
				return -1;
			}
		}
		else if(strcmp(p, "p2") == 0)
		{
			p = strtok(NULL, "&=");
			isData.dbgi = atoi(p);
			if(isData.dbgi < 0 || isData.dbgi > 10)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p3"))
		{
			p = strtok(NULL, "&=");
			isData.btntoggle = p[0];
			if(isData.btntoggle != 'E' && isData.btntoggle != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p4"))
		{
			p = strtok(NULL, "&=");
			isData.btnnorrev = p[0];
			if(isData.btnnorrev != 'E' && isData.btnnorrev != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p5"))
		{
			p = strtok(NULL, "&=");
			isData.pressure = p[0];
			if(isData.pressure != 'E' && isData.pressure != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p6"))
		{
			p = strtok(NULL, "&=");
			isData.distimern = p[0];
			if(isData.distimern != 'E' && isData.distimern != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p7"))
		{
			p = strtok(NULL, "&=");
			isData.distimert = p[0];
			if(isData.distimert != 'E' && isData.distimert != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p8"))
		{
			p = strtok(NULL, "&=");
			isData.distimerr = p[0];
			if(isData.distimerr != 'E' && isData.distimerr != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p9"))
		{
			p = strtok(NULL, "&=");
			isData.distime = atoi(p);
			if(isData.distime < 0 || isData.distime > 120)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p10"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowm = p[0];
			if(isData.disthrowm != 'E' && isData.disthrowm != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p11"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowmt = atoi(p);
			if(isData.disthrowmt < 0 || isData.disthrowmt > 14400)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p12"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowd = p[0];
			if(isData.disthrowd != 'E' && isData.disthrowd != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p13"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowdt = atoi(p);
			if(isData.disthrowdt < 0 || isData.disthrowdt > 14400)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p14"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowp1 = p[0];
			if(isData.disthrowp1 != 'E' && isData.disthrowp1 != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p15"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowp1t = atoi(p);
			if(isData.disthrowp1t < 0 || isData.disthrowp1t > 14400)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p16"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowp2 = p[0];
			if(isData.disthrowp2 != 'E' && isData.disthrowp2 != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p17"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowp2t = atoi(p);
			if(isData.disthrowp2t < 0 || isData.disthrowp2t > 14400)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p18"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowp3 = p[0];
			if(isData.disthrowp3 != 'E' && isData.disthrowp3 != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p19"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowp3t = atoi(p);
			if(isData.disthrowp3t < 0 || isData.disthrowp3t > 14400)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p20"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowp4 = p[0];
			if(isData.disthrowp4 != 'E' && isData.disthrowp4 != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p21"))
		{
			p = strtok(NULL, "&=");
			isData.disthrowp4t = atoi(p);
			if(isData.disthrowp4t < 0 || isData.disthrowp4t > 14400)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p22"))
		{
			p = strtok(NULL, "&=");
			isData.track_id1 = atof(p);
			sscanf(p, "%lf", &isData.track_id1);
			if(isData.track_id1 <= 0 || isData.track_id1 > 2)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p23"))
		{
			p = strtok(NULL, "&=");
			isData.track_id2 = atof(p);
			sscanf(p, "%lf", &isData.track_id2);
			if(isData.track_id2 <= 0 || isData.track_id2 > 2)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p24"))
		{
			p = strtok(NULL, "&=");
			isData.track_id3 = atof(p);
			sscanf(p, "%lf", &isData.track_id3);
			if(isData.track_id3 <= 0 || isData.track_id3 > 2)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p25"))
		{
			p = strtok(NULL, "&=");
			isData.track_id4 = atof(p);
			sscanf(p, "%lf", &isData.track_id4);
			if(isData.track_id4 <= 0 || isData.track_id4 > 2)
			{
				return -1;
			}
		}
		p = strtok(NULL, "&=");
	}

	memcpy(&cwData.isData, &isData, sizeof(INPUT_STANDARD_DATA));
	return 1;
}

/*******************************************************************************
  Class :
  Function Name : saveAuxiliaryInput
  Input         :
  Output        :
  purpose       : save auxiliary input config
  Note          :
*******************************************************************************/
short saveAuxiliaryInput()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p1"))
		{
			p = strtok(NULL, "&=");
			iaData.aps = p[0];
			if(iaData.aps != 'N' && iaData.aps != 'S' && iaData.aps != 'D')
			{
				return -1;
			}
		}
		else if(strcmp(p, "p2") == 0)
		{
			p = strtok(NULL, "&=");
			iaData.dbgi = atoi(p);
			if(iaData.dbgi < 0 || iaData.dbgi > 10)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p4"))
		{
			p = strtok(NULL, "&=");
			iaData.btntoggle = p[0];
			if(iaData.btntoggle != 'E' && iaData.btntoggle != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p3"))
		{
			p = strtok(NULL, "&=");
			iaData.btnnormal = p[0];
			if(iaData.btnnormal != 'E' && iaData.btnnormal != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p5"))
		{
			p = strtok(NULL, "&=");
			iaData.btnreverse = p[0];
			if(iaData.btnreverse != 'E' && iaData.btnreverse != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p6"))
		{
			p = strtok(NULL, "&=");
			iaData.distimern = p[0];
			if(iaData.distimern != 'E' && iaData.distimern != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p7"))
		{
			p = strtok(NULL, "&=");
			iaData.distimert = p[0];
			if(iaData.distimert != 'E' && iaData.distimert != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p8"))
		{
			p = strtok(NULL, "&=");
			iaData.distimerr = p[0];
			if(iaData.distimerr != 'E' && iaData.distimerr != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p9"))
		{
			p = strtok(NULL, "&=");
			iaData.distime = atoi(p);
			if(iaData.distime < 0 || iaData.distime > 120)
			{
				return -1;
			}
		}
		p = strtok(NULL, "&=");
	}

	memcpy(&cwData.iaData, &iaData, sizeof(INPUT_AUXILIARY_DATA));
	return 1;
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
short saveOutput()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p1"))
		{
			p = strtok(NULL, "&=");
			opData.lights = p[0];
			if(opData.lights != 'E' && opData.lights != 'D')
			{
				return -1;
			}
		}
		else if(strcmp(p, "p2") == 0)
		{
			p = strtok(NULL, "&=");
			opData.pinf = p[0];
			if(opData.pinf != 'N' && opData.pinf != 'S' && opData.pinf != 'A')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p3"))
		{
			p = strtok(NULL, "&=");
			opData.duration = atoi(p);
			if(opData.duration < 0 || opData.duration > 30)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p4"))
		{
			p = strtok(NULL, "&=");
			opData.flashrate = atoi(p);
			if(opData.flashrate <= 0 || opData.flashrate > 120)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p5"))
		{
			p = strtok(NULL, "&=");
			opData.strobe = p[0];
			if(opData.strobe != 'C' && opData.strobe != 'F')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p6"))
		{
			p = strtok(NULL, "&=");
			opData.stroberate = atoi(p);
			if(opData.stroberate <= 0 || opData.stroberate > 120)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p7"))
		{
			p = strtok(NULL, "&=");
			opData.gapthrow = p[0];
			if(opData.gapthrow != 'E' && opData.gapthrow != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p8"))
		{
			p = strtok(NULL, "&=");
			opData.gapphase = p[0];
			if(opData.gapphase != 'E' && opData.gapphase != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p9"))
		{
			p = strtok(NULL, "&=");
			opData.strobe_gapped = p[0];
			if(opData.strobe_gapped != 'E' && opData.strobe_gapped != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p10"))
		{
			p = strtok(NULL, "&=");
			opData.strobe_out_phase = p[0];
			if(opData.strobe_out_phase != 'E' && opData.strobe_out_phase != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p11"))
		{
			p = strtok(NULL, "&=");
			opData.track_pos_rate = atoi(p);
			if(opData.track_pos_rate <= 0 || opData.track_pos_rate > 120)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p12"))
		{
			p = strtok(NULL, "&=");
			opData.track_off = p[0];
			if(opData.track_off != 'E' && opData.track_off != 'D')
			{
				return -1;
			}
		}
		p = strtok(NULL, "&=");
	}

	memcpy(&cwData.opData, &opData, sizeof(OUTPUT_DATA));
	return 1;
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
short saveTimer()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p1"))
		{
			p = strtok(NULL, "&=");
			trData.srai = p[0];
			if(trData.srai != 'D' && trData.srai != 'E')
			{
				return -1;
			}
		}
		else if(strcmp(p, "p2") == 0)
		{
			p = strtok(NULL, "&=");
			trData.srdi = p[0];
			if(trData.srdi != 'E' && trData.srdi != 'F' && trData.srdi != 'G' &&
				trData.srdi != 'H' && trData.srdi != 'B' && trData.srdi != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p3"))
		{
			p = strtok(NULL, "&=");
			trData.srf = p[0];
			if(trData.srf != 'N' && trData.srf != 'R')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p4"))
		{
			p = strtok(NULL, "&=");
			trData.srtime = atoi(p);
			if(trData.srtime < 0 || trData.srtime > 1900)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p5"))
		{
			p = strtok(NULL, "&=");
			sscanf(p, "%lf", &trData.psdebounce);
			if(trData.psdebounce < 0 || trData.psdebounce > 30)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p6"))
		{
			p = strtok(NULL, "&=");
			sscanf(p, "%lf", &trData.psbreaktime);
			if(trData.psbreaktime < 0 || trData.psbreaktime > 2)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p7"))
		{
			p = strtok(NULL, "&=");
			trData.psintertime = atof(p);
			sscanf(p, "%lf", &trData.psintertime);
			if(trData.psintertime < 0 || trData.psintertime > 2)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p8"))
		{
			p = strtok(NULL, "&=");
			trData.inputdebounce = atoi(p);
			if(trData.inputdebounce < 30 || trData.inputdebounce > 100)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p9"))
		{
			p = strtok(NULL, "&=");
			trData.maxrtime = atoi(p);
			if(trData.maxrtime < 2 || trData.maxrtime > 10)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p10"))
		{
			p = strtok(NULL, "&=");
			trData.dvcdtime = atoi(p);
			if(trData.dvcdtime < 20 || trData.dvcdtime > 300)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p11"))
		{
			p = strtok(NULL, "&=");
			trData.ldttime = atoi(p);
			if(trData.ldttime < 1 || trData.ldttime > 20)
			{
				return -1;
			}
		}
		p = strtok(NULL, "&=");
	}

	memcpy(&cwData.trData, &trData, sizeof(TIMER_DATA));
	return 1;
}

/*******************************************************************************
  Class :
  Function Name : saveSpecial
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
short saveSpecial()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p1"))
		{
			p = strtok(NULL, "&=");
			sfData.salone = p[0];
			if(sfData.salone != 'E' && sfData.salone != 'D')
			{
				return -1;
			}
		}
		else if(strcmp(p, "p2") == 0)
		{
			p = strtok(NULL, "&=");
			sfData.swstatus = p[0];
			if(sfData.swstatus != 'S' && sfData.swstatus != 'A' && sfData.swstatus != 'B')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p3"))
		{
			p = strtok(NULL, "&=");
			sfData.cpd = p[0];
			if(sfData.cpd != 'O' && sfData.cpd != 'A' && sfData.cpd != 'B')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p4"))
		{
			p = strtok(NULL, "&=");
			sfData.aureturn = p[0];
			if(sfData.aureturn != 'E' && sfData.aureturn != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p5"))
		{
			p = strtok(NULL, "&=");
			sscanf(p, "%lf", &sfData.bcaution);
			if(sfData.bcaution < 0 || sfData.bcaution > 12)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p6"))
		{
			p = strtok(NULL, "&=");
			sscanf(p, "%lf", &sfData.bdisable);
			if(sfData.bdisable < 0 || sfData.bdisable > 12)
			{
				return -1;
			}
		}
		p = strtok(NULL, "&=");
	}

	memcpy(&cwData.sfData, &sfData, sizeof(SPECT_FUNCTION_DATA));
	return 1;
}

/*******************************************************************************
  Class :
  Function Name : getAllConfig
  Input         :
  Output        :
  purpose       : get all config
  Note          :
*******************************************************************************/
void getAllConfig()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}
}

/*******************************************************************************
  Class :
  Function Name : getAllConfig
  Input         :
  Output        :
  purpose       : save all config
  Note          :
*******************************************************************************/
short saveIO()
{
	int index = 0;
	int tempInt = 0;
	char a[224][50];
	int i,j;

	char *tmp;
	tmp = strtok(data, "para=");
	tmp = strtok(NULL, "para=");
	tmp = strtok(NULL, "para=");
	tmp = strtok(NULL, "para=");

	char *result;
	result = strtok(tmp, "||");
	while(result != NULL)
	{
		if(strstr(result, "-999") != NULL)
		{
			tempInt = atoi(result);
			if(tempInt != -999)
			{
				strcpy(a[index], result);
			}
		}
		else
		{
			strcpy(a[index], result);
		}

		result = strtok(NULL, "||");
		index++;
	}

	//save io
	index = 0;
	for(i = 0; i < MAX_NUM_IO; i++)
	{
		ioData[i].pinNo = (int8_t)atoi(a[index]);
		index++;

		sprintf(ioData[i].devName, "%s", a[index]);
		if(strlen(ioData[i].devName) > 30)
		{
			return -2;
		}
		index++;

		ioData[i].devID = (int16_t)atoi(a[index]);
		if(ioData[i].devID <= 0 || ioData[i].devID > 200)
		{
			return -3;
		}
		index++;

		ioData[i].devType = *(a[index] + 0);
		if(ioData[i].devType < 65 || ioData[i].devType > 90)
		{
			return -4;
		}
		index++;

		ioData[i].devSubType = *(a[index] + 0);
		if(ioData[i].devSubType < 65 || ioData[i].devSubType > 90)
		{
			return -5;
		}
		index++;

		ioData[i].devHighLow = toupper(*(a[index] + 0));
		if(ioData[i].devHighLow != 'L' && ioData[i].devHighLow != 'H')
		{
			return -6;
		}
		index++;

		ioData[i].devIoType = *(a[index] + 0);
		index++;
	}

	memcpy(&cwData.ioData, &ioData, sizeof(IO_DEF_DATA) * MAX_NUM_IO);
	return 1;
}

/*******************************************************************************
  Class :
  Function Name : saveCommFunc
  Input         :
  Output        :
  purpose       : 
  Note          :
*******************************************************************************/
short saveCommFunc()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p1"))
		{
			p = strtok(NULL, "&=");
			commData.type = p[0];
			if(commData.type != 'U' && commData.type != 'T' && commData.type != 'S' && commData.type != 'B')
			{
				return -1;
			}
		}
		else if(strcmp(p, "p2") == 0)
		{
			p = strtok(NULL, "&=");
			commData.baudRate = atoi(p);
			if(commData.baudRate <= 0 || commData.baudRate > 460800)
			{
				return -1;
			}
			else
			{
				if((fp1 = fopen("/home/flexyard/flexyard/sysconfig/config_serial_port.txt", "w")) == NULL)
				{
					return -1;
				}
				else
				{
					memset(tempStr, 0x0, sizeof(char) * 20);
					sprintf(tempStr, "%d", commData.baudRate);
					int tempLen = 0;
					while(1 == 1)
					{
						if(tempStr[tempLen] && tempStr[tempLen] != '\0')
						{
							tempLen++;
						}
						else
						{
							break;
						}
						if(tempLen >= 20)
						{
							return -1;
						}
					}
					if(fwrite(&tempStr, tempLen, 1, fp1) != 1)
					{
						fclose(fp1);
						return -1;
					}
				}
				fclose(fp1);
			}
		}
		else if(0 == strcmp(p, "p3"))
		{
			p = strtok(NULL, "&=");
			commData.modeRep = p[0];
			if(commData.modeRep != 'E' && commData.modeRep != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p4"))
		{
			p = strtok(NULL, "&=");
			commData.modeTra = p[0];
			if(commData.modeTra != 'E' && commData.modeTra != 'D')
			{
				return -1;
			}
		}
		else if(strcmp(p, "p5") == 0)
		{
			p = strtok(NULL, "&=");
			commData.nodeID = atoi(p);
			if(commData.nodeID <= 0 || commData.nodeID >= 1000)
			{
				return -1;
			}
		}
		else if(strcmp(p, "p6") == 0)
		{
			p = strtok(NULL, "&=");
			commData.netID = atoi(p);
			if(commData.netID <= 0 || commData.netID >= 1000)
			{
				return -1;
			}
		}
		else if(strcmp(p, "p7") == 0)
		{
			p = strtok(NULL, "&=");
			commData.railroadID = atoi(p);
			if(commData.railroadID <= 0 || commData.railroadID >= 100)
			{
				return -1;
			}
		}
		else if(strcmp(p, "p8") == 0)
		{
			p = strtok(NULL, "&=");
			commData.yardID = atoi(p);
			if(commData.yardID <= 0 || commData.yardID >= 100)
			{
				return -1;
			}
		}
		else if(strcmp(p, "p9") == 0)
		{
			p = strtok(NULL, "&=");
			commData.subtype = p[0];
			if(commData.subtype != 'A' && commData.subtype != 'B' && commData.subtype != 'C')
			{
				return -1;
			}
		}
		p = strtok(NULL, "&=");
	}

	memcpy(&cwData.commData, &commData, sizeof(COMM_FUNC_DATA));
	return 1;
}

/*******************************************************************************
  Class :
  Function Name : saveSwtypeFunc
  Input         :
  Output        :
  purpose       : 
  Note          :
*******************************************************************************/
short saveSwtypeFunc()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p1"))
		{
			p = strtok(NULL, "&=");
			swtypeData.type = p[0];
			if(swtypeData.type != 'A' && swtypeData.type != 'B' &&
				swtypeData.type != 'C' && swtypeData.type != 'D')
			{
				return -1;
			}
		}
		p = strtok(NULL, "&=");
	}

	memcpy(&cwData.swtypeData, &swtypeData, sizeof(SWITCH_TYPE_FUNC_DATA));
	return 1;
}

/*******************************************************************************
  Class :
  Function Name : saveRadioFunc
  Input         :
  Output        :
  purpose       : 
  Note          :
*******************************************************************************/
short saveRadioFunc()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p1"))
		{
			p = strtok(NULL, "&=");
			radioData.enable = p[0];
			if(radioData.enable != 'E' && radioData.enable != 'D')
			{
				return -1;
			}
			//printf("radioData.enable=%c\n", radioData.enable);
		}
		else if(0 == strcmp(p, "p9"))
		{
			p = strtok(NULL, "&=");
			radioData.LPO = p[0];
			if(radioData.LPO != 'A' && radioData.LPO != 'B' && radioData.LPO != 'C')
			{
				return -1;
			}
			//printf("radioData.LPO=%c\n", radioData.LPO);
		}
		else if(strcmp(p, "p10") == 0)
		{
			p = strtok(NULL, "&=");
			radioData.TRBT = atoi(p);
			if(radioData.TRBT <= 0 || radioData.TRBT > 30)
			{
				return -1;
			}
			//printf("radioData.TRBT=%d\n", radioData.TRBT);
		}
		else if(0 == strcmp(p, "p11"))
		{
			p = strtok(NULL, "&=");
			sscanf(p, "%lf", &radioData.TTID);
			if(radioData.TTID < 0 || radioData.TTID > 2.5)
			{
				return -1;
			}
			//printf("radioData.TTID=%f\n", radioData.TTID);
		}
		else if(strcmp(p, "p12") == 0)
		{
			p = strtok(NULL, "&=");
			radioData.TRCL = atoi(p);
			if(radioData.TRCL < 0 || radioData.TRCL > 60)
			{
				return -1;
			}
			//printf("radioData.TRCL=%d\n", radioData.TRCL);
		}
		else if(strcmp(p, "p13") == 0)
		{
			p = strtok(NULL, "&=");
			radioData.TPCL = atoi(p);
			if(radioData.TPCL < 0 || radioData.TPCL > 60)
			{
				return -1;
			}
			//printf("radioData.TPCL=%d\n", radioData.TPCL);
		}
		else if(0 == strcmp(p, "p14"))
		{
			p = strtok(NULL, "&=");
			radioData.enableAC = p[0];
			if(radioData.enableAC != 'E' && radioData.enableAC != 'D')
			{
				return -1;
			}
			//printf("radioData.enableAC=%c\n", radioData.enableAC);
		}
		else if(strcmp(p, "p15") == 0)
		{
			p = strtok(NULL, "&=");
			radioData.ACTBP = atoi(p);
			if(radioData.ACTBP <= 0 || radioData.ACTBP > 150)
			{
				return -1;
			}
			//printf("radioData.ACTBP=%d\n", radioData.ACTBP);
		}
		else if(strcmp(p, "p16") == 0)
		{
			p = strtok(NULL, "&=");
			radioData.ACRT = atoi(p);
			if(radioData.ACRT < 0 || radioData.ACRT > 300)
			{
				return -1;
			}
			//printf("radioData.ACRT=%d\n", radioData.ACRT);
		}
		p = strtok(NULL, "&=");
	}

	memcpy(&cwData.radioData, &radioData, sizeof(RADIO_FUNC_DATA));
	return 1;
}

/*******************************************************************************
  Class :
  Function Name : saveDTMFInput
  Input         :
  Output        :
  purpose       : save DTMF input config
  Note          :
*******************************************************************************/
short saveDTMFInput()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p4"))
		{
			p = strtok(NULL, "&=");
			idData.btntoggle = p[0];
			if(idData.btntoggle != 'E' && idData.btntoggle != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p3"))
		{
			p = strtok(NULL, "&=");
			idData.btnnormal = p[0];
			if(idData.btnnormal != 'E' && idData.btnnormal != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p5"))
		{
			p = strtok(NULL, "&=");
			idData.btnreverse = p[0];
			if(idData.btnreverse != 'E' && idData.btnreverse != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p6"))
		{
			p = strtok(NULL, "&=");
			idData.distimern = p[0];
			if(idData.distimern != 'E' && idData.distimern != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p7"))
		{
			p = strtok(NULL, "&=");
			idData.distimert = p[0];
			if(idData.distimert != 'E' && idData.distimert != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p8"))
		{
			p = strtok(NULL, "&=");
			idData.distimerr = p[0];
			if(idData.distimerr != 'E' && idData.distimerr != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p9"))
		{
			p = strtok(NULL, "&=");
			idData.distime = atoi(p);
			if(idData.distime < 0 || idData.distime > 200)
			{
				return -1;
			}
		}
		p = strtok(NULL, "&=");
	}

	memcpy(&cwData.idData, &idData, sizeof(INPUT_DTMF_DATA));
	return 1;
}

/*******************************************************************************
  Class :
  Function Name : saveAudio
  Input         :
  Output        :
  purpose       : save audio config
  Note          :
*******************************************************************************/
short saveAudio()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p1"))
		{
			p = strtok(NULL, "&=");
			aoData.dur_dtmf_nor = atoi(p);
			if(aoData.dur_dtmf_nor < 0 || aoData.dur_dtmf_nor > 5)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p2"))
		{
			p = strtok(NULL, "&=");
			aoData.dur_dtmf_rev = atoi(p);
			if(aoData.dur_dtmf_rev < 0 || aoData.dur_dtmf_rev > 5)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p3"))
		{
			p = strtok(NULL, "&=");
			aoData.dur_dtmf_gap = atoi(p);
			if(aoData.dur_dtmf_gap < 0 || aoData.dur_dtmf_gap > 5)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p4"))
		{
			p = strtok(NULL, "&=");
			aoData.dur_aux_1 = atoi(p);
			if(aoData.dur_aux_1 < 0 || aoData.dur_aux_1 > 5)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p5"))
		{
			p = strtok(NULL, "&=");
			aoData.dur_aux_2 = atoi(p);
			if(aoData.dur_aux_2 < 0 || aoData.dur_aux_2 > 5)
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p6"))
		{
			p = strtok(NULL, "&=");
			aoData.dur_aux_3 = atoi(p);
			if(aoData.dur_aux_3 < 0 || aoData.dur_aux_3 > 5)
			{
				return -1;
			}
		}
		p = strtok(NULL, "&=");
	}

	memcpy(&cwData.aoData, &aoData, sizeof(AUDIO_DATA));
	return 1;
}

/*******************************************************************************
  Class :
  Function Name : saveLightout
  Input         :
  Output        :
  purpose       : save lightout config
  Note          :
*******************************************************************************/
short saveLightout()
{
	char *p;

	p = strtok(data, "&=");
	while(p != NULL)
	{
		if(0 == strcmp(p, "p1"))
		{
			p = strtok(NULL, "&=");
			lightoutData.enable = p[0];
			if(lightoutData.enable != 'E' && lightoutData.enable != 'D')
			{
				return -1;
			}
		}
		else if(0 == strcmp(p, "p2"))
		{
			p = strtok(NULL, "&=");
			lightoutData.caution = atoi(p);
			if(lightoutData.caution < 0 || lightoutData.caution > 500)
			{
				return -1;
			}
		}
		p = strtok(NULL, "&=");
	}

	FILE *fp;
	short dataLen = sizeof(SPECT_LIGHTOUT_DATA);
	if((fp = fopen("/home/flexyard/flexyard/sysconfig/lightout.dat", "w")) == NULL)
	{
		return -1;
	}
	else
	{
		if(fwrite((char *)&lightoutData, dataLen, 1, fp) != 1)
		{
			fclose(fp);
			return -1;
		}
	}
	fclose(fp);
	return 1;
}

